#include "boss_fight_ui.hpp"
#include "easings.hpp"

namespace gui {
  BossFightUI::BossFightUI(GameLevel level)
    : $level(level)
  {
    $status.position(0, 0, 300, SCREEN_HEIGHT);
    $status.layout(
        "[main_status]"
        "[(150)status_3|(150)status_4]"
        "[(150)status_5|(150)status_6]"
        "[(150)status_7|(150)status_8]"
        );

    $overlay.position(300, 0, SCREEN_WIDTH - 300, SCREEN_HEIGHT);
    $overlay.layout("[overlay_1|overlay_2|overlay_4]"
    "[overlay_5|overlay_6|overlay_8]"
    "[overlay_9|overlay_10|overlay_12]"
    "[overlay_13|overlay_14|overlay_16]");

    $boss_image = textures::get("boss_fight");
    auto bounds = $boss_image.sprite->getLocalBounds();
    $boss_image.sprite->setPosition({300 + bounds.size.x / 2, bounds.size.y / 2});
    $boss_image.sprite->setOrigin({bounds.size.x / 2, bounds.size.y / 2});
  }

  void BossFightUI::init() {
    $status.world().set_the<Background>({$status.$parser});

    for(auto& [name, cell] : $status.cells()) {
      auto button = $status.entity(name);
      $status.set<Rectangle>(button, {});
      $status.set<Clickable>(button, {
          [this, name](auto, auto){ fmt::println("STATUS: {}", name); }
      });
      if(name == "main_status") {
        $status.set<Textual>(button, {fmt::format("HP: {}", $boss_hp)});
      } else {
        $status.set<Label>(button, {"Attack"});
      }
    }
    $status.init();

    for(auto& [name, cell] : $overlay.cells()) {
      auto region = $overlay.entity(name);
      $overlay.set<Clickable>(region, {
          [this, name](auto, auto){ fmt::println("OVERLAY: {}", name); }
      });
    }

    $overlay.init();
  }

  void BossFightUI::bounce_boss(sf::RenderWindow& window) {
    auto time = $clock.getElapsedTime();
    float tick = ease::out_bounce(ease::sine(time.asSeconds()));
    float scale = std::lerp(1.0, 1.15, tick);
    $boss_image.sprite->setScale({scale, scale});
    window.draw(*$boss_image.sprite);
  }

  void BossFightUI::render(sf::RenderWindow& window) {
    if($boss_hit) {
      bounce_boss(window);
    } else {
      window.draw(*$boss_image.sprite);
    }

    if($boss_hp == 0) {
      $overlay.show_label("overlay_1", "YOU WON!");
      $overlay.show_label("overlay_4", "CLICK TO CONTINUE...");
    }

    $status.render(window);
    $overlay.render(window);
  }

  bool BossFightUI::mouse(float x, float y) {
    if($status.mouse(x, y)) {
      fmt::println("STATUS");
    }

    if($overlay.mouse(x, y)) {
      $boss_hit = !$boss_hit;
      $boss_hp--;
    }

    return false;
  }

  void BossFightUI::update_level(GameLevel &level) {
    $level = level;
    $boss_hp = 10 * $level.index + 1;  // make him stronger
    $boss_hit = false;
    $overlay.close<Label>("overlay_1");
    $overlay.close<Label>("overlay_4");
    init();
  }
}
